PROGRAM Comm_TP3;                                                               {                            }
                                                                                {                            }
{                           Turbo Pascal (Version 3)                          } {-This is the comment column }
{                                                                             } { which explains the program }
{         Comm_TP3.PAS Ver. 1.01 - RS-232 Support for IBM Compatibles         } { operation where necessary. }
{                             (c) Copyright, 1989                             } {                            }
{                              Kevin R. Bulgrien                              } { It is likely to cause your }
{                              February 01, 1989                              } { hard copy to exceed a line }
{                                                                             } { width of 80 characters, so }
{ This program works on Turbo Pascal 3.0.  Instructions are provided in the   } { your printer will need to  }
{ file Comm_TP3.DOC.  Comm_TP4, Comm_TP5, and Comm_TC2 are also available.    } { be set for smaller print.  }
{                                                                             } {                            }
{ Written by:  Kevin R. Bulgrien           Version 1.01 completed 02/01/89    } { I hope this is acceptably  }
{                                                                             } { inconvenient as I believe  }
{ Send all correspondance to:              Kevin R. Bulgrien                  } { that documentation is far  }
{                                          P. O. Box 563                      } { easier to read when it is  }
{ Internet: kbulgrien@worldnet.att.net     Longview, TX 75606                 } { not mingled with the code. }
{           http://home.att.net/~kbulgrien (903) 758-4035                     } {                            }
{                                                                             } { No code should be placed   }
{ Also contact at: c/o  K. A. Bulgrien, 1700 Sheldon Road, Sandusky, MI 48471 } { in this column so that it  }
{                                                                             } { will be easier to remove.  }
{                                                                             } { easier to read when it is  }
{ Version History                                                             } { done like this...          }
{                                                                             } {                            }
{  1.00, 11/88 Original code uploaded to GEnie's Borland Roundtable           } {                            }
{  1.01, 02/89 Bug found & fixed in Procedure RemoveInt. Port [$21] was ANDed } {                            }
{              with $08 or $10 instead of ORed with those values.             } {                            }
                                                                                {                            }
{$C-} {$U-} { Do NOT use ^C to halt programs which install interrupt drivers  } {                            }
            { because you would bypass the code to uninstall the drivers. As  } {                            }
            { a result, they will be left in RAM which DOS thinks is unused.  } {                            }
            { (Because the program terminated)  This becomes major potential  } {                            }
            { for a system CRASH!                                             } {                            }
                                                                                {                            }
CONST                                                                           {                            }
  Data_Seg : INTEGER = 0;                                                       {-DS Register for interrupt  }
  MaxSize = 511;                                                                {-Maximum input buffer size  }
                                                                                {                            }
TYPE                                                                            {-8250 Communications Chip   }
  INS8250 = RECORD                                                              { -------------------------- }
              THR : INTEGER;                                                    { Transmit Holding Register  }
              RHR : INTEGER;                                                    { Receive Holding Register   }
              DLL : INTEGER;                                                    { Divisor Latch Register LSB }
              IER : INTEGER;                                                    { Interrupt Enable Register  }
              DLM : INTEGER;                                                    { Divisor Latch Register MSB }
              IIR : INTEGER;                                                    { Interrupt ID Register      }
              LCR : INTEGER;                                                    { Line Control Register      }
              MCR : INTEGER;                                                    { Modem Control Register     }
              LSR : INTEGER;                                                    { Line Status Register       }
              MSR : INTEGER;                                                    { Modem Status Register      }
            END;                                                                {                            }
                                                                                {                            }
  Registers = RECORD                                                            {-8088/86 Registers type for }
                CASE INTEGER OF                                                 { any DOS/BIOS calls used in }
                  0 : (AX,BX,CX,DX,BP,SI,DI,DS,ES, Flags : INTEGER);            { these routines.            }
                  1 : (AL,AH,BL,BH,CL,CH,DL,DH : BYTE);                         {                            }
              END;                                                              {                            }
                                                                                {                            }
  ComSettingsRecord = RECORD                                                    {-Used to hold the current   }
                       Baud : BYTE;                                             { settings of COM1 or COM2   }
                       Parity : BYTE;                                           {                            }
                       Stop : BYTE;                                             {                            }
                       Bits : BYTE;                                             {                            }
                     END;                                                       {                            }
                                                                                {                            }
  ComSettingsType = ARRAY [1..2] OF ComSettingsRecord;                          {-COM1 & COM2 Settings       }
                                                                                {                            }
  BaudType = (B110,B150,B300,B600,B1200,B2400,B4800,B9600,B19200,B38400);       {-Baud rates supported       }
                                                                                {                            }
  ParityType = (None, Odd, Null, Even, MarkOff, Mark, SpaceOff, Space);         {-Parity types supported     }
                                                                                {                            }
  ComBuffersType = ARRAY [1..2, 0..MaxSize] OF BYTE;                            {-The input buffers for COM1 }
                                                                                { and COM2                   }
  OutBuffer = STRING [255];                                                     {                            }
                                                                                {                            }
CONST                                                                           {                            }
  RS232 : ARRAY [1..2] OF INS8250 = ( ( THR:$3F8; RHR:$3F8; DLL:$3F8;           {-COM1 addresses of the 8250 }
                                        IER:$3F9; DLM:$3F9; IIR:$3FA;           { registers so that they may }
                                        LCR:$3FB; MCR:$3FC; LSR:$3FD;           { be accessed by name.       }
                                        MSR:$3FE ),                             {                            }
                                      ( THR:$2F8; RHR:$2F8; DLL:$2F8;           {-COM2 addresses of the 8250 }
                                        IER:$2F9; DLM:$2F9; IIR:$2FA;           { registers so that they may }
                                        LCR:$2FB; MCR:$2FC; LSR:$2FD;           { be accessed by name        }
                                        MSR:$2FE ) );                           {                            }
                                                                                {                            }
VAR                                                                             {                            }
  OldIntSeg, OldIntOfs : ARRAY [1..2] OF INTEGER;                               {-Original COMx int. vectors }
  InHead, InTail : ARRAY [1..2] OF INTEGER;                                     {-Input buffer pointers      }
  IntInstalled : ARRAY [1..2] OF BOOLEAN;                                       {-TRUE if interrupt in place }
  Carrier : ARRAY [1..2] OF BOOLEAN;                                            {-TRUE if Carrier detected   }
  ComSettings : ComSettingsType;                                                {-COM1 & COM2 line settings  }
  InBuffer : ComBuffersType;                                                    {-Input circular queue buffer}
  CurrentCom : BYTE;                                                            {-COM port currently logged  }
  Regs : Registers;                                                             {-8088/86 CPU Registers      }
  MaxPorts : BYTE;                                                              {-Number of Valid COM ports  }
  IntCom : BYTE;                                                                {-Interrupt handler temporary}
                                                                                {                            }
{ This procedure sets up the selected COM port to the specified parameters.  The Com parameter specifies the }
{ port to set up.  It must be in the range 1 to 2, and is checked for errors.  The Baud parameter is must be }
{ in the range 0 to 9, and is not range checked.  The TYPE BaudTable is included only to document the baud   }
{ rates supported in BaudTable.  I.E. BaudTable [1] refers to 150 Baud.  It is unneccessary for any other    }
{ purpose.  In the same manner, ParityType is provide to document the parity settings allowed.  Use ORD() to }
{ get the correct BYTE value to pass:  ORD(B110) returns the BYTE that selects 110 baud and ORD(None) gives  }
{ the BYTE that selects no parity. (Use global declarations of these TYPEs for these examples to work.)  1.5 }
{ stop bits are used when StopBits = 2 AND DataBits = 5, but otherwise StopBits will set the correct number  }
{ of stop bits in the range 1 to 2.  DataBits may be set with 5 to 8 for the number of data bits to use.     }
{                                                                                                            }
{ Mark parity means that parity is enabled and the parity bit is always set to 0.  Space parity means that   }
{ parity is enabled and the parrity bit is always set to 1.  MarkOff and SpaceOff indicate that Mark or Space}
{ parity is chosen but parity is disabled.  Functionally they are equivalent to NONE - as is NULL.           }
                                                                                {                            }
PROCEDURE SetupRS232 (Com, Baud, Parity, StopBits, DataBits : BYTE);            {                            }
CONST                                                                           {-These values set the baud  }
  BaudTable : ARRAY [0..9] OF INTEGER = ($0417, $0300, $0180, $00C0, $0060,     { rate of the 8250 when they }
                                         $0030, $0018, $000C, $0006, $0003);    { are written to DLL & DMM.  }
TYPE                                                                            {                            }
  BaudType = (B110,B150,B300,B600,B1200,B2400,B4800,B9600,B19200,B38400);       {-Baud rates supported       }
  ParityType = (None,Odd,Null,Even,MarkOff,Mark,SpaceOff,Space);                {-Parity settings supported  }
                                                                                {                            }
VAR                                                                             {-Temporary variable to hold }
  Parameters : BYTE;                                                            { correct LCR register value }
BEGIN                                                                           {                            }
  IF (Com <= MaxPorts)                                                          {-Check validity of Com      }
    THEN BEGIN                                                                  {                            }
           INLINE ($FA);                                                        {-Always when writing to 8250}
           PORT [RS232 [Com].LCR] := PORT [RS232 [Com].LCR] OR $80;             {-Allow access to DLL & DLM  }
           PORT [RS232 [Com].DLL] := LO (BaudTable [Baud]);                     {-Set the baud rate          }
           PORT [RS232 [Com].DLM] := HI (BaudTable [Baud]);                     {                            }
           Parameters := (DataBits - 5) AND $03;                                {-Build the value to write   }
           Parameters := Parameters OR (((StopBits - 1) SHL 2) AND $04);        { to Line Control Register.  }
           Parameters := Parameters OR ((Parity SHL 3) AND $38);                {                            }
           PORT [RS232 [Com].LCR] := Parameters;                                {-Set parity, data/stop bits }
           INLINE ($FB);                                                        {-Done writing to 8250 regs. }
         END                                                                    {                            }
    ELSE BEGIN                                                                  {                            }
           WRITELN (#13,#10, 'Error!  COM', Com, ' not available', #10);        {-Mostly here for debugging  }
         END;                                                                   { purposes                   }
END;                                                                            {                            }
                                                                                {                            }
{ This procedure handles all interrupts from the 8250 communications chip.  All interrupt types are provided }
{ for even though I only implemented the receive data interrupt and crudely used the modem status interrupt. }
{ The skeleton is there though, so you can write your own implementations for the interrupts.  Incoming data }
{ is stored in InBuffer if the buffer is not full - otherwise it is ignored. The buffer is full when (InTail }
{ [IntCom] + 1) MOD (MaxSize + 1) = InHead [IntCom].  The buffer is empty when InTail [IntCom] = InHead      }
{ [IntCom].  InTail [IntCom] is incremented so that it always points to where the next item will be put.     }
{ Modem (port) status is monitored for Carrier Detect.  The global BOOLEAN variable 'Carrier' always shows   }
{ the status of each COM port if its interrupt handler is active.  TRUE = Carrier Detected and FALSE = No    }
{ Carrier Detected.  This is done for programs that will use a modem for input.  NOTE:  Stack checking must  }
{ always be turned off in interrupt handlers ($K-).                                                          }
                                                                                {                            }
PROCEDURE IntHandler;                                                           {                            }
BEGIN                                                                           {                            }
  INLINE ($9C/               { PUSHF               }                            {-This Inline code is needed }
          $50/               { PUSH  AX            }                            { for Turbo Pascal version 3 }
          $53/               { PUSH  BX            }                            { in order to use procedures }
          $51/               { PUSH  CX            }                            { as interrupt handlers. Its'}
          $52/               { PUSH  DX            }                            { incomplete in that it does }
          $56/               { PUSH  SI            }                            { not provide for local vars }
          $57/               { PUSH  DI            }                            { inside the procedure. This }
          $1E/               { PUSH  DS            }                            { is why IntCom is declared  }
          $06/               { PUSH  ES            }                            { as a global variable.      }
          $2E/               { SEG   CS:           }                            {                            }
          $A1/Data_Seg/      { MOV   AX,[Data_Seg] }                            {                            }
          $8E/$D8);          { MOV   DS,AX         }                            {                            }
                                                                                {                            }
  PORT [$20] := $0B;                                                            {-Allow access to 8259 ISR   }
  IntCom := 3 - ((PORT [$20] AND $18) SHR 3);                                   {-Detect interrupting port   }
  CASE PORT [RS232 [IntCom].IIR] AND $06 OF                                     {                            }
    0 : BEGIN                                                                   {-Modem Status Change Int.   }
          Carrier [IntCom] := ($80 AND PORT [RS232 [IntCom].MSR] = $80);        {-Save status of Carrier     }
        END;                                                                    {                            }
    2 : BEGIN                                                                   {-Transmit Register Empty    }
        END;                                                                    {                            }
    4 : BEGIN                                                                   {-Receive Register Full      }
          PORT [RS232 [IntCom].LCR] := PORT [RS232 [IntCom].LCR] AND $7F;       {-Allow THR,RBR & IER access }
          IF (InTail [IntCom] + 1) MOD (MaxSize + 1) <> InHead [IntCom]         {                            }
            THEN BEGIN                                                          {-If the buffer is not full, }
                   InBuffer [IntCom,InTail[IntCom]] := PORT [RS232[IntCom].RHR];{ add the character and set  }
                   InTail [IntCom] := (InTail [IntCom] + 1) MOD (MaxSize + 1);  { the queue buffer pointer   }
                 END                                                            {                            }
            ELSE BEGIN                                                          {-If the buffer is full, the }
                   IF (PORT [RS232 [IntCom].RHR] = $00) THEN { DO Nothing };    { data is read & not stored  }
                 END;                                                           {                            }
        END;                                                                    {                            }
    6 : BEGIN                                                                   {-Line Status change & Error }
        END;                                                                    {                            }
  END;                                                                          {                            }
  PORT [$20] := $20;                                                            {-Notify 8259 that interrupt }
                                                                                {                            }
  INLINE ($07/               { POP   ES    }                                    {-This inline code is needed }
          $1F/               { POP   DS    }                                    { in Turbo Pascal Version 3  }
          $5F/               { POP   DI    }                                    { to end procedures that are }
          $5E/               { POP   SI    }                                    { used as interrupt handlers }
          $5A/               { POP   DX    }                                    {                            }
          $59/               { POP   CX    }                                    {                            }
          $5B/               { POP   BX    }                                    {                            }
          $58/               { POP   AX    }                                    {                            }
          $9D/               { POPF        }                                    {                            }
          $8B/$E5/           { MOV   SP,BP }                                    {                            }
          $5D/               { POP   BP    }                                    {                            }
          $CF);              { IRET        }                                    {                            }
END;                                                                            {                            }
                                                                                {                            }
{ This procedure installs and enables the specified serial port interrupt.  It also forces the appropriate   }
{ input buffer to the empty state.  Carrier is initialized to the current state of the Carrier Detect line   }
{ on the port.  The old serial port interrupt vector is saved so that it can be reinstalled when we remove   }
{ our serial port interrupt.  DTR and RTS are forced to the ready state and the 8250 interrupts are enabled  }
{ by writing $0B to the MCR.  To enable all four types of 8250 interrupts, write $0F to the IER.  (I enabled }
{ the Modem Status Change and Receive Buffer Full interrupts by writing $09 to the IER). ORing $EF with PORT }
{ [$21] enables IRQ4 (COM1), while ORing $F7 enables IRQ3 (COM2).  Hardware interrupts should be disabled    }
{ during the installation process since the 8250 & 8259 ports are being accessed.                            }
                                                                                {                            }
PROCEDURE InstallInt (Com : BYTE);                                              {                            }
BEGIN                                                                           {                            }
  IF NOT IntInstalled [Com] AND (Com <= MaxPorts)                               {-Don't install the handler  }
    THEN BEGIN                                                                  { twice or if nonexistant.   }
           INLINE ($FA);                                                        {                            }
           Data_Seg := DSEG;                                                    {-Info for interrupt handler }
           InTail [Com] := 0;                                                   {-Set input buffer to empty  }
           InHead [Com] := 0;                                                   {                            }
           Carrier [Com] := ($80 AND PORT [RS232 [Com].MSR] = $80);             {-Read Carrier Detect status }
           PORT [RS232 [Com].LCR] := PORT [RS232 [Com].LCR] AND $7F;            {-Allow THR,RBR & IER access }
           PORT [RS232 [Com].IER] := $00;                                       {-Disable 8250 interrupts    }
           IF PORT [RS232 [Com].LSR] <> 0 THEN { Nothing };                     {-Reset interrupts that were }
           IF PORT [RS232 [Com].RHR] <> 0 THEN { Nothing };                     { waiting to be processed    }
           WITH Regs DO                                                         {                            }
             BEGIN                                                              {                            }
               AH := $35;                                                       {-DOS function call gets the }
               AL := $0D - Com;                                                 { original interrupt vector  }
               MSDOS (Regs);                                                    {                            }
               OldIntSeg [Com] := ES;                                           {-Store vector for later use }
               OldIntOfs [Com] := DX;                                           {                            }
               AH := $25;                                                       {-DOS function call installs }
               AL := $0D - Com;                                                 { new interrupt vector       }
               DS := CSEG;                                                      {                            }
               DX := OFS (IntHandler);                                          {                            }
               MSDOS (Regs);                                                    {                            }
             END;                                                               {                            }
           IntInstalled [Com] := TRUE;                                          {-The interrupt is installed }
           CASE Com OF                                                          {                            }
             1 : PORT [$21] := PORT [$21] AND $EF;                              {-Enable 8259 IRQ4 handling  }
             2 : PORT [$21] := PORT [$21] AND $F7;                              {-Enable 8259 IRQ3 handling  }
           END;                                                                 {                            }
           PORT [RS232 [Com].LCR] := PORT [RS232 [Com].LCR] AND $7F;            {-Allow THR,RBR & IER access }
           PORT [RS232 [Com].IER] := $09;                                       {-Enable 8250 interrupts     }
           PORT [RS232 [Com].MCR] := $0B;                                       {-Set DTR & RTS so the other }
           INLINE ($FB);                                                        { device knows we are ready  }
         END                                                                    { to recieve data            }
    ELSE BEGIN                                                                  {                            }
           WRITE (#13, #10, 'Error!  COM', Com, ' ');                           {                            }
           IF IntInstalled [Com]                                                {                            }
             THEN WRITELN ('interrupt already installed',#10)                   {-Mostly here for debugging  }
             ELSE WRITELN ('not available',#10)                                 { purposes.  Remove in your  }
         END;                                                                   { final program if you want. }
END;                                                                            {                            }
                                                                                {                            }
{ This procedure removes the specified serial port interrupt and reinstalls the original interrupt vectors.  }
{ DTR & RTS are set OFF and 8250 interrupts are disabled by writing $00 to the MCR. All 8250 interrupt types }
{ are disabled by writing $00 to the IER.  ORing $10 with PORT [$21] disables IRQ4 (COM1), while ORing $08   }
{ disables IRQ3 (COM2).  Hardware interrupts must be disabled during this process.                           }
                                                                                {                            }
PROCEDURE RemoveInt (Com : BYTE);                                               {                            }
BEGIN                                                                           {                            }
  IF IntInstalled [Com]                                                         {-Don't remove if interrupt  }
    THEN BEGIN                                                                  { has not been installed     }
           INLINE ($FA);                                                        {                            }
           CASE Com OF                                                          {                            }
             1 : PORT [$21] := PORT [$21] OR $10;                               {-Disable 8259 IRQ4 handling }
             2 : PORT [$21] := PORT [$21] OR $08;                               {-Disable 8259 IRQ3 handling }
           END;                                                                 {                            }
           PORT [RS232 [Com].LCR] := PORT [RS232 [Com].LCR] AND $7F;            {-Allow THR,RBR & IER access }
           PORT [RS232 [Com].IER] := $00;                                       {-Disable 8250 interrupts    }
           PORT [RS232 [Com].MCR] := $00;                                       {-Set DTR & RTS off          }
           WITH Regs DO                                                         {                            }
             BEGIN                                                              {                            }
               AH := $25;                                                       {-DOS function call restores }
               AL := $0D - Com;                                                 { original interrupt vector  }
               DS := OldIntSeg [Com];                                           {                            }
               DX := OldIntOfs [Com];                                           {                            }
               MSDOS (Regs);                                                    {                            }
             END;                                                               {                            }
           IntInstalled [Com] := FALSE;                                         {-The interrupt is removed   }
           INLINE ($FB);                                                        {                            }
         END                                                                    {                            }
    ELSE BEGIN                                                                  {-Mostly here for debugging  }
           WRITE (#13, #10, 'Error!  COM', Com, ' ');                           { purposes.  Remove in your  }
           WRITELN ('interrupt is not installed', #10);                         { program if you wish.       }
         END;                                                                   {                            }
END;                                                                            {                            }
                                                                                {                            }
{ This procedure writes character or string data to the serial port.  It does this by directly reading and   }
{ writing to the 8250 communications chip.  This is an example that may be modified to suit your purposes.   }
{ As is, it pauses the program while it sends the data.  If it cannot send a character after 65535 tries, it }
{ aborts the sending process.  This could easily be converted to a FUNCTION that returns the BOOLEAN value   }
{ TimeOut.  The statement: (PORT [RS232 [Com].LSR] AND $20) <> $20 indicates when the THR is ready for a new }
{ character to send.  CTS and DSR are not checked, but if you want to check for them (PORT [RS232 [Com].MSR] }
{ AND $30) must equal $30.  Interrupts must be disabled while using the 8250 port registers.                 }
                                                                                {                            }
PROCEDURE WriteCOM (Com : BYTE; Data : OutBuffer);                              {                            }
VAR                                                                             {                            }
 LoopVar,                                                                       {-Pointer to output char     }
 TimeLoop : INTEGER;                                                               {-Timeout counter variable   }
 TimeOut : BOOLEAN;                                                             {-True if unable to send     }
BEGIN                                                                           {                            }
  IF IntInstalled [Com]                                                         {                            }
    THEN BEGIN                                                                  {                            }
           LoopVar := 0;                                                        {                            }
           TimeOut := FALSE;                                                    {                            }
           WHILE (LoopVar < LENGTH (Data)) AND NOT TimeOut DO                   {-Send the data one char at  }
             BEGIN                                                              { a time unless the port was }
               TimeLoop := 0;                                                   { timed out.                 }
               LoopVar := LoopVar + 1;                                          {                            }
               WHILE (TimeLoop<32767)AND((PORT[RS232[Com].LSR]AND$20)<>$20) DO  {-Do not try to send data if }
                 TimeLoop := TimeLoop + 1;                                      { the THR is not empty yet.  }
               IF TimeLoop <> 32767                                             {                            }
                 THEN BEGIN                                                     {                            }
                        INLINE ($FA);                                           {                            }
                        PORT [RS232[Com].LCR] := PORT [RS232 [Com].LCR] AND $7F;{-Allow THR,RBR & IER access }
                        PORT [RS232[Com].THR] := ORD (Data [LoopVar]);          {-Put the data to send in    }
                        INLINE ($FB);                                           { the THR                    }
                      END                                                       {                            }
                 ELSE BEGIN                                                     {                            }
                        TimeOut := TRUE;                                        {-WriteCOM aborts if the THR }
                        WRITELN (#13,#10, 'Timeout on COM', Com);               { takes too long to become   }
                      END;                                                      { empty so you can send more }
             END;                                                               { data                       }
         END                                                                    {                            }
    ELSE BEGIN                                                                  {                            }
           WRITELN (#13,#10, 'COM', Com, ' not installed');                     {                            }
         END;                                                                   {                            }
END;                                                                            {                            }
                                                                                {                            }
{ This function is an example of how to get a character from the serial port. As is, if the buffer is empty, }
{ it waits until a character arrives, so this will not work for the TTY emulation. The interrupts are always }
{ disabled when the buffer pointers are checked or modified.  Beware!  Do not completely disable interrupts  }
{ in the wait loop or else you never will get a character if there is not one there already.                 }
                                                                                {                            }
FUNCTION ReadCOM (Com : BYTE) : CHAR;                                           {                            }
VAR                                                                             {                            }
  CharReady : BOOLEAN;                                                          {-TRUE if there is data in   }
BEGIN                                                                           { the input buffer           }
  IF IntInstalled [Com]                                                         {                            }
    THEN BEGIN                                                                  {                            }
           CharReady := FALSE;                                                  {                            }
           REPEAT                                                               {-Wait for data to arrive    }
             INLINE ($FA);                                                      {                            }
             CharReady := InTail [Com] <> InHead [Com];                         {-Check to  see if buffer is }
             INLINE ($FB);                                                      { empty                      }
           UNTIL CharReady;                                                     {                            }
           INLINE ($FA);                                                        {                            }
           ReadCOM := CHR (InBuffer [Com, InHead [Com]]);                       {-Read a character of data   }
           InHead [Com] := (InHead [Com] + 1) MOD (MaxSize + 1);                {-Update the buffer pointer  }
           INLINE ($FB);                                                        {                            }
         END                                                                    {                            }
    ELSE BEGIN                                                                  {                            }
           WRITELN (#13,#10, 'COM', Com, ' not installed');                     {                            }
         END;                                                                   {                            }
END;                                                                            {                            }
                                                                                {                            }
{ End of RS-232 handler routines ----------- Start of TTY emulation routines }  { Example TTY program starts }
                                                                                {                            }
{ A crude but effective procedure to allow the user to change settings of a COM port in use.  CurrentCom and }
{ ComSettings are used to determine how the port is currently set up.  As you change the parameters, these   }
{ are also updated.  Once again, keep in mind that the object of this program is not to provide a glamorous  }
{ terminal program.  Rather it serves as a simple model for those wanting to incorporate serial routines in  }
{ their own programs.                                                                                        }
                                                                                {                            }
PROCEDURE SetUpPort (Com : BYTE);                                               {                            }
VAR                                                                             {                            }
  ResetPort : BOOLEAN;                                                          {-TRUE when settings changed }
  InkeyChr : CHAR;                                                              {-Keyboard input variable    }
BEGIN                                                                           {                            }
  WRITELN;                                                                      {                            }
  ResetPort := FALSE;                                                           {                            }
  WRITELN ('COM', Com, ' Setup', #10);                                          {-Select a baud rate         }
  WRITELN ('0)  110           5)  2400');                                       {                            }
  WRITELN ('1)  150           6)  4800');                                       {-Note that defaults are     }
  WRITELN ('2)  300           7)  9600');                                       { allowed if you press <CR>  }
  WRITELN ('3)  600           8) 19200');                                       { at any of the prompts. The }
  WRITELN ('4) 1200           9) 38400', #10);                                  { port is not reset unless   }
  WRITE ('Select a baud rate [', ComSettings [Com] . Baud, ']: ');              { the defaults are changed.  }
  REPEAT                                                                        {                            }
    READ (KBD, InkeyChr);                                                       {                            }
  UNTIL (InkeyChr IN ['0'..'9', #13]);                                          {                            }
  WRITELN (InkeyChr, #10);                                                      {                            }
  IF (InkeyChr <> #13)                                                          {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [Com] . Baud := ORD (InkeyChr) - 48;                     {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {-Select a parity setting    }
  WRITELN ('0) None           2) None');                                        {                            }
  WRITELN ('1) Odd            3) Even', #10);                                   {                            }
  WRITE ('Select a parity type [', ComSettings [Com] . Parity, ']: ');          {                            }
  REPEAT                                                                        {                            }
    READ (KBD, InkeyChr);                                                       {                            }
  UNTIL (InkeyChr IN ['0'..'3', #13]);                                          {                            }
  WRITELN (InkeyChr, #10);                                                      {                            }
  IF (InkeyChr <> #13)                                                          {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [Com] . Parity := ORD (InkeyChr) - 48;                   {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  WRITE ('Select number of stop bits [', ComSettings [Com] . Stop, ']: ');      {-Select number of stop bits }
  REPEAT                                                                        {                            }
    READ (KBD, InkeyChr);                                                       {                            }
  UNTIL (InkeyChr IN ['1'..'2', #13]);                                          {                            }
  WRITELN (InkeyChr, #10);                                                      {                            }
  IF (InkeyChr <> #13)                                                          {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [Com] . Stop := ORD (InkeyChr) - 48;                     {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  WRITE ('Select number of data bits [', ComSettings [Com] . Bits, ']: ');      {-Select number of data bits }
  REPEAT                                                                        {                            }
    READ (KBD, InkeyChr);                                                       {                            }
  UNTIL (InkeyChr IN ['5'..'8', #13]);                                          {                            }
  WRITELN (InkeyChr, #10);                                                      {                            }
  IF (InkeyChr <> #13)                                                          {                            }
    THEN BEGIN                                                                  {                            }
           ComSettings [Com] . Bits := ORD (InkeyChr) - 48;                     {                            }
           ResetPort := TRUE;                                                   {                            }
         END;                                                                   {                            }
  IF ResetPort                                                                  {-If the settings changed,   }
    THEN SetupRS232 (Com, ComSettings [Com] . Baud,                             { reset the port             }
                          ComSettings [Com] . Parity,                           {                            }
                          ComSettings [Com] . Stop,                             {                            }
                          ComSettings [Com] . Bits);                            {                            }
END;                                                                            {                            }
                                                                                {                            }
{ This provides a simple terminal emulation that might be used to prove that these routines really work, and }
{ that they are not hard to use.  I got to playing, and perhaps it got a bit more complex than necessary...  }
{ but then again, who said it had to be quick and dirty.  The LocalEcho parameter determines if characters   }
{ typed on the keyboard should be echoed to the screen.                                                      }
                                                                                {                            }
PROCEDURE TTY (LocalEcho : BOOLEAN);                                            {                            }
VAR                                                                             {                            }
  ExitTTY,                                                                      {-TRUE when ready to quit    }
  DataReady : BOOLEAN;                                                          {-TRUE if buffer not empty   }
  OldCarrier : ARRAY [1..2] OF BOOLEAN;                                         {-Helps detect Carrier change}
  Buffer : CHAR;                                                                {-A character buffer         }
BEGIN                                                                           {                            }
  OldCarrier [1] := NOT Carrier [1];                                            {-Forces Carrier status to   }
  OldCarrier [2] := NOT Carrier [2];                                            { be displayed               }
  DataReady := FALSE;                                                           {-Initialize everything      }
  ExitTTY := FALSE;                                                             {                            }
  Buffer := #0;                                                                 {                            }
  CLRSCR;                                                                       {                            }
  WRITELN ('Terminal emulator commands', #10);                                  {-Brief summary of command   }
  WRITELN ('<ALT C>  Toggle Port in use COM1/COM2');                            { keys that can be used      }
  WRITELN ('<Alt E>  Toggle Local Echo On/Off');                                {                            }
  WRITELN ('<Alt P>  Change Port Parameters');                                  {                            }
  WRITELN ('<Alt X>  Exit');                                                    {                            }
  REPEAT                                                                        {-Terminal emulation starts  }
    INLINE ($FA);                                                               {                            }
    DataReady := (InTail [CurrentCom] <> InHead [CurrentCom]);                  {-If data has been received, }
    INLINE ($FB);                                                               { print one character        }
    IF DataReady                                                                {                            }
      THEN BEGIN                                                                { CHR(12) is interpreted as  }
             INLINE ($FA);                                                      { a FormFeed, and so clears  }
             Buffer := CHR(InBuffer [CurrentCom, InHead [CurrentCom]]);         { the screen                 }
             InHead [CurrentCom] := (InHead [CurrentCom] + 1) MOD (MaxSize + 1);{                            }
             INLINE ($FB);                                                      { Input buffer is updated    }
             CASE Buffer OF                                                     {                            }
               #12 : CLRSCR;                                                    {                            }
               ELSE  WRITE (Buffer);                                            {                            }
             END;                                                               {                            }
           END;                                                                 {                            }
    IF (OldCarrier [CurrentCom] <> Carrier [CurrentCom])                        {-If a change in carrier     }
      THEN BEGIN                                                                { detect occurs, notify the  }
             WRITELN;                                                           { user of the new status     }
             IF Carrier [CurrentCom]                                            {                            }
               THEN WRITELN ('CARRIER DETECTED (COM', CurrentCom, ')')          {                            }
               ELSE WRITELN ('NO CARRIER (COM', CurrentCom, ')');               {                            }
             OldCarrier [CurrentCom] := Carrier [CurrentCom];                   {                            }
           END;                                                                 {                            }
    IF KEYPRESSED                                                               {-If a key has been pressed, }
      THEN BEGIN                                                                { process it                 }
             READ (KBD, Buffer);                                                {                            }
             IF (Buffer = #27) AND KEYPRESSED                                   {-Extended key codes require }
               THEN BEGIN                                                       { another read               }
                      READ (KBD, Buffer);                                       {                            }
                      CASE Buffer OF                                            {                            }
                        #46 : IF (1 + ORD (CurrentCom = 1)) <= MaxPorts         {-<ALT C> lets you toggle    }
                                THEN BEGIN                                      { between ports if the new   }
                                       CurrentCom := 1 + ORD (CurrentCom = 1);  { port exists                }
                                       WRITELN (#13,#10, 'COM', CurrentCom);    {                            }
                                     END                                        {                            }
                                ELSE BEGIN                                      {                            }
                                       WRITE (#13,#10, 'COM');                  {                            }
                                       WRITE (1 + ORD (CurrentCom = 1));        {                            }
                                       WRITELN (' not available');              {                            }
                                     END;                                       {                            }
                        #18 : LocalEcho := NOT LocalEcho;                       {-<ALT E> toggles Local Echo }
                        #25 : SetupPort (CurrentCom);                           {-<ALT P> allows port setup  }
                        #45 : ExitTTY := TRUE;                                  {-<ALT X> exits the program  }
                        ELSE  WriteCOM (CurrentCom, CHR(27) + Buffer);          {-Other extended key codes   }
                      END;                                                      { are sent to the port       }
                    END                                                         {                            }
               ELSE BEGIN                                                       {-Normal key codes are sent  }
                      CASE Buffer OF                                            { or translated and sent     }
                        #12 : BEGIN                                             {                            }
                                WriteCOM (CurrentCom, Buffer);                  {-FormFeed clears screen if  }
                                IF LocalEcho THEN CLRSCR;                       { local echo is on           }
                              END;                                              {                            }
                        #13 : BEGIN                                             {-A carriage return also     }
                                WriteCOM (CurrentCom, Buffer + CHR(10));        { sends a line feed          }
                                IF LocalEcho THEN WRITELN;                      {                            }
                              END;                                              {                            }
                        ELSE  BEGIN                                             {-All other characters are   }
                                WriteCOM (CurrentCom, Buffer);                  { sent as typed              }
                                IF LocalEcho THEN WRITE (Buffer);               {                            }
                              END;                                              {                            }
                      END;                                                      {                            }
                    END;                                                        {                            }
           END;                                                                 {                            }
  UNTIL ExitTTY;                                                                {-Continue emulation until   }
END;                                                                            { <ALT X> is pressed.        }
                                                                                {                            }
FUNCTION Equipment : INTEGER;                                                   {-This function returns what }
BEGIN                                                                           { equipment is present on    }
  INTR ($11, Regs);                                                             { the machine it is running  }
  Equipment := Regs.AX;                                                         { on.                        }
END;                                                                            {                            }
                                                                                {                            }
BEGIN                                                                           {                            }
  MaxPorts := (Equipment AND $0E00) SHR 9;                                      {-Find # of system COM ports }
  IntInstalled [1] := FALSE;                                                    {-No interrupt handlers are  }
  IntInstalled [2] := FALSE;                                                    { installed on start up      }
  ComSettings [1] . Baud := ORD (B9600);                                        {-Define COM1 default setup  }
  ComSettings [1] . Parity := ORD (None);                                       {                            }
  ComSettings [1] . Stop := 1;                                                  {                            }
  ComSettings [1] . Bits := 8;                                                  {                            }
  ComSettings [2] . Baud := ORD (B2400);                                        {-Define COM2 default setup  }
  ComSettings [2] . Parity := ORD (None);                                       {                            }
  ComSettings [2] . Stop := 1;                                                  {                            }
  ComSettings [2] . Bits := 8;                                                  {                            }
  IF (MaxPorts >= 1)                                                            {                            }
    THEN BEGIN                                                                  {                            }
           SetupRS232 (1, ComSettings [1] . Baud,                               {-Initialize COM1 to the     }
                          ComSettings [1] . Parity,                             { default setup              }
                          ComSettings [1] . Stop,                               {                            }
                          ComSettings [1] . Bits);                              {                            }
           InstallInt (1);                                                      {-Set up the COM1 interrupt  }
         END                                                                    { if the computer has a port }
    ELSE WRITELN ('Error!  No serial ports installed in this computer');        {                            }
  IF (MaxPorts >= 2)                                                            {                            }
    THEN BEGIN                                                                  {                            }
           SetupRS232 (2, ComSettings [2] . Baud,                               {                            }
                          ComSettings [2] . Parity,                             {-Initialize COM2 to the     }
                          ComSettings [2] . Stop,                               { default setup              }
                          ComSettings [2] . Bits);                              {                            }
           InstallInt (2);                                                      {-Set up the COM2 interrupt  }
         END;                                                                   {                            }
  CurrentCom := 1;                                                              {-Set COM1 as logged port    }
  TTY (FALSE);                                                                  {-TTY with local echo off    }
  IF IntInstalled [1]                                                           {-Uninstall any interrupts   }
    THEN RemoveInt (1);                                                         { that are installed         }
  IF IntInstalled [2]                                                           {                            }
    THEN RemoveInt (2);                                                         {                            }
END.                                                                            {                            }
